------------------------------------------------------------------------------------------------------
-- Replace the old (outdated) blizzard API
------------------------------------------------------------------------------------------------------
local function GetNumSkillLines() return 6 end
local function GetSkillLineInfo(skillIndex)
local name,_,level="",0,0 local i = select(skillIndex,GetProfessions()) if i ~= nil then name,_,level = GetProfessionInfo(i) end return name,nil,nil,level end
------------------------------------------------------------------------------------------------------
-- Declare the main addon variable
------------------------------------------------------------------------------------------------------
ProfessionBars = LibStub("AceAddon-3.0"):NewAddon("ProfessionBars", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")

------------------------------------------------------------------------------------------------------
-- Actions scheduled when we leave combat - table
------------------------------------------------------------------------------------------------------
ProfessionBars.LeaveCombatActions = {}

------------------------------------------------------------------------------------------------------
-- Execute an action or (if we are in combat) schedule it to be executed on leaving combat
------------------------------------------------------------------------------------------------------
function ProfessionBars:ExecuteFrameAction(frame, action, args)
	if InCombatLockdown() then
		ProfessionBars.LeaveCombatActions[#ProfessionBars.LeaveCombatActions+1] = {
			frame = frame,
			action = action,
			args = args,
		}
		return
	end
	if type(args) == "function" then
		args = args()
	end
	if type(args) == "table" then
		frame[action](frame, unpack(args))
	else
		frame[action](frame, args)
	end
end

------------------------------------------------------------------------------------------------------
-- Execute actions that are scheduled to be executed on leaving combat
------------------------------------------------------------------------------------------------------
function ProfessionBars:DoScheduledActions()
	if not (#ProfessionBars.LeaveCombatActions >= 1) then return end
	for num, data in pairs(ProfessionBars.LeaveCombatActions) do
		if type(data.args) == "function" then
			data.args = data.args()
		end
		if type(data.args) == "table" then
			data.frame[data.action](data.frame, unpack(data.args))
		else
			data.frame[data.action](data.frame, data.args)
		end
		ProfessionBars.LeaveCombatActions[num] = nil
	end
end

------------------------------------------------------------------------------------------------------
-- Various variables we need
------------------------------------------------------------------------------------------------------
profbars_temp = {};
local db
local jc_bar
local jc_movebutton
local insc_bar
local insc_movebutton
local lw_bar
local lw_movebutton
local ench_bar
local ench_movebutton
local bs_bar
local bs_movebutton

------------------------------------------------------------------------------------------------------
-- Debugging slash command
------------------------------------------------------------------------------------------------------
local profbars_debugoptions = {
	name = "ProfessionBarsBlacklist",
	handler = ProfessionBars,
	type = 'group',
	args = {
		add = {
			type = 'input',
			name = 'BlacklistAdd',
			desc = 'Add an item to the disenchanting blacklist. Arguments: <itemlink>',
			set = 'AddBlacklist',
			get = nil,
		},
		remove = {
			type = 'input',
			name = 'BlacklistRemove',
			desc = 'Remove an item from the disenchanting blacklist. Arguments: <itemlink>',
			set = 'RemoveBlacklist',
			get = nil,
		},
	},
}
------------------------------------------------------------------------------------------------------
-- Option defaults
------------------------------------------------------------------------------------------------------
local defaults = {
	profile = {
		mode = 2,
		help = true,
		offsets = {
			jc = {
				x = 250,
				y = 250
			},
			insc = {
				x = 250,
				y = 250
			},
			lw = {
				x = 250,
				y = 250,
			},
			ench = {
				x = 250,
				y = 250,
			},
			bs = {
				x = 250,
				y = 250,
			}
		},
		blacklist = {},
	},
}

------------------------------------------------------------------------------------------------------
-- Table of all our needed frames
------------------------------------------------------------------------------------------------------
local profbars_frames = {
	-- Jewelcrafting frames
	profbars_jcbutton1	= {item = 23425,	type = "jc",	offsetx = 210,	offsety = 0,	texturepath = "Interface\\Icons\\INV_Ore_Adamantium"},
	profbars_jcbutton2	= {item = 36909,	type = "jc",	offsetx = 245,	offsety = 0,	texturepath = "Interface\\Icons\\INV_Ore_Cobalt"},
	profbars_jcbutton3	= {item = 36912,	type = "jc",	offsetx = 280,	offsety = 0,	texturepath = "Interface\\Icons\\INV_Ore_Saronite_01"},
	profbars_jcbutton4	= {item = 23424,	type = "jc",	offsetx = 175,	offsety = 0,	texturepath = "Interface\\Icons\\INV_Ore_FelIron"},
	profbars_jcbutton5	= {item = 10620,	type = "jc",	offsetx = 140,	offsety = 0,	texturepath = "Interface\\Icons\\INV_Ore_Thorium_02"},
	profbars_jcbutton6	= {item = 3858,		type = "jc",	offsetx = 105,	offsety = 0,	texturepath = "Interface\\Icons\\INV_Ore_Mithril_02"},
	profbars_jcbutton7	= {item = 2772,		type = "jc",	offsetx = 70,	offsety = 0,	texturepath = "Interface\\Icons\\INV_Ore_Iron_01"},
	profbars_jcbutton8	= {item = 2771,		type = "jc",	offsetx = 35,	offsety = 0,	texturepath = "Interface\\Icons\\INV_Ore_Tin_01"},
	profbars_jcbutton9	= {item = 2770,		type = "jc",	offsetx = 0,	offsety = 0,	texturepath = "Interface\\Icons\\INV_Ore_Copper_01"},
	profbars_jcbutton10 = {item = 36910,	type = "jc",	offsetx = 315,	offsety = 0,	texturepath = "Interface\\Icons\\inv_ore_platinum_01"},
	
	-- Inscription frames
	profbars_inscbutton1	= {item = 39342,	type = "insc",	offsetx = 210,	offsety = 0,	texturepath = "Interface\\Icons\\inv_inscription_pigment_nether"},
	profbars_inscbutton2	= {item = 39343,	type = "insc",	offsetx = 245,	offsety = 0,	texturepath = "Interface\\Icons\\inv_inscription_pigment_azure"},
	--profbars_inscbutton3	= {item = 0,		type = "insc",	offsetx = 0,	offsety = 0,	texturepath = ""},
	profbars_inscbutton4	= {item = 39341,	type = "insc",	offsetx = 175,	offsety = 0,	texturepath = "Interface\\Icons\\inv_inscription_pigment_silvery"},
	profbars_inscbutton5	= {item = 39340,	type = "insc",	offsetx = 140,	offsety = 0,	texturepath = "Interface\\Icons\\inv_inscription_pigment_violet"},
	profbars_inscbutton6	= {item = 39339,	type = "insc",	offsetx = 105,	offsety = 0,	texturepath = "Interface\\Icons\\inv_inscription_pigment_emerald"},
	profbars_inscbutton7	= {item = 39338,	type = "insc",	offsetx = 70,	offsety = 0,	texturepath = "Interface\\Icons\\inv_inscription_pigment_golden"},
	profbars_inscbutton8	= {item = 39334,	type = "insc",	offsetx = 35,	offsety = 0,	texturepath = "Interface\\Icons\\inv_inscription_pigment_grey"},
	profbars_inscbutton9	= {item = 39151,	type = "insc",	offsetx = 0,	offsety = 0,	texturepath = "Interface\\Icons\\inv_inscription_pigment_white"},
	
	-- Leatherworking frames
	profbars_lwbutton1 = {item = 23793,		type = "lw",	offsetx = 210,	offsety = 0,	texturepath = "Interface\\Icons\\inv_misc_leatherscrap_11"},
	profbars_lwbutton2 = {item = 33568,		type = "lw",	offsetx = 245,	offsety = 0,	texturepath = "Interface\\Icons\\inv_misc_leatherscrap_15"},
	profbars_lwbutton3 = {item = 38425,		type = "lw",	offsetx = 280,	offsety = 0,	texturepath = "Interface\\Icons\\inv_misc_leatherscrap_19"},
	profbars_lwbutton4 = {item = 21887,		type = "lw",	offsetx = 175,	offsety = 0,	texturepath = "Interface\\Icons\\inv_misc_leatherscrap_10"},
	profbars_lwbutton5 = {item = 8170,		type = "lw",	offsetx = 140,	offsety = 0,	texturepath = "Interface\\Icons\\inv_misc_leatherscrap_02"},
	profbars_lwbutton6 = {item = 4304,		type = "lw",	offsetx = 105,	offsety = 0,	texturepath = "Interface\\Icons\\inv_misc_leatherscrap_08"},
	profbars_lwbutton7 = {item = 4234,		type = "lw",	offsetx = 70,	offsety = 0,	texturepath = "Interface\\Icons\\inv_misc_leatherscrap_07"},
	profbars_lwbutton8 = {item = 2319,		type = "lw",	offsetx = 35,	offsety = 0,	texturepath = "Interface\\Icons\\inv_misc_leatherscrap_05"},
	profbars_lwbutton9 = {item = 2318,		type = "lw",	offsetx = 0,	offsety = 0,	texturepath = "Interface\\Icons\\inv_misc_leatherscrap_03"},
	profbars_lwbutton10 = {item = 4231,		type = "lw",	offsetx = 0,	offsety = -35,	texturepath = "Interface\\Icons\\inv_misc_pelt_wolf_01"},
	profbars_lwbutton11 = {item = 4233,		type = "lw",	offsetx = 35,	offsety = -35,	texturepath = "Interface\\Icons\\inv_misc_pelt_bear_02"},
	profbars_lwbutton12 = {item = 4236,		type = "lw",	offsetx = 70,	offsety = -35,	texturepath = "Interface\\Icons\\inv_misc_pelt_wolf_02"},
	profbars_lwbutton13 = {item = 8172,		type = "lw",	offsetx = 105,	offsety = -35,	texturepath = "Interface\\Icons\\inv_misc_pelt_bear_01"},
	profbars_lwbutton14 = {item = 15407,	type = "lw",	offsetx = 140,	offsety = -35,	texturepath = "Interface\\Icons\\inv_misc_pelt_bear_03"},
	
	-- Blacksmithing frames
	profbars_bsbutton1 = {item = 2840,	type = "bs",	offsetx = 0,	offsety = 0,	texturepath = "Interface\\Icons\\inv_ingot_02",	spell = 2657},
	profbars_bsbutton2 = {item = 3576,	type = "bs",	offsetx = 35,	offsety = 0,	texturepath = "Interface\\Icons\\inv_ingot_05",	spell = 3304},
	profbars_bsbutton3 = {item = 2841,	type = "bs",	offsetx = 70,	offsety = 0,	texturepath = "Interface\\Icons\\inv_ingot_bronze",	spell = 2659},
	profbars_bsbutton4 = {item = 2842,	type = "bs",	offsetx = 105,	offsety = 0,	texturepath = "Interface\\Icons\\inv_ingot_01",	spell = 2658},
	profbars_bsbutton5 = {item = 3575,	type = "bs",	offsetx = 140,	offsety = 0,	texturepath = "Interface\\Icons\\inv_ingot_iron",	spell = 3307},
	profbars_bsbutton6 = {item = 3577,	type = "bs",	offsetx = 175,	offsety = 0,	texturepath = "Interface\\Icons\\inv_ingot_03",	spell = 3308},
	profbars_bsbutton7 = {item = 3859,	type = "bs",	offsetx = 210,	offsety = 0,	texturepath = "Interface\\Icons\\inv_ingot_steel",	spell = 3569},
	profbars_bsbutton8 = {item = 3860,	type = "bs",	offsetx = 245,	offsety = 0,	texturepath = "Interface\\Icons\\inv_ingot_06",	spell = 10097},
	profbars_bsbutton9 = {item = 6037,	type = "bs",	offsetx = 280,	offsety = 0,	texturepath = "Interface\\Icons\\inv_ingot_08",	spell = 10098},
	profbars_bsbutton10 = {item=11371,	type = "bs",	offsetx = 0,	offsety = -35,	texturepath = "Interface\\Icons\\inv_ingot_mithril",	spell = 14891},
	profbars_bsbutton11 = {item=12359,	type = "bs",	offsetx = 35,	offsety = -35,	texturepath = "Interface\\Icons\\inv_ingot_07",	spell = 16153},
	profbars_bsbutton12 = {item=17771,	type = "bs",	offsetx = 70,	offsety = -35,	texturepath = "Interface\\Icons\\inv_ingot_thorium",	spell = 22967},
	profbars_bsbutton13 = {item=23445,	type = "bs",	offsetx = 105,	offsety = -35,	texturepath = "Interface\\Icons\\inv_ingot_feliron",	spell = 29356},
	profbars_bsbutton14 = {item=23446,	type = "bs",	offsetx = 140,	offsety = -35,	texturepath = "Interface\\Icons\\inv_ingot_10",	spell = 29358},
	profbars_bsbutton15 = {item=23447,	type = "bs",	offsetx = 175,	offsety = -35,	texturepath = "Interface\\Icons\\inv_ingot_11",	spell = 29359},
	profbars_bsbutton16 = {item=23448,	type = "bs",	offsetx = 210,	offsety = -35,	texturepath = "Interface\\Icons\\inv_ingot_felsteel",	spell = 29360},
	profbars_bsbutton17 = {item=23449,	type = "bs",	offsetx = 245,	offsety = -35,	texturepath = "Interface\\Icons\\inv_ingot_09",	spell = 29361},
	profbars_bsbutton18 = {item=23573,	type = "bs",	offsetx = 280,	offsety = -35,	texturepath = "Interface\\Icons\\inv_ingot_adamantite",	spell = 29686},
	profbars_bsbutton19 = {item=35128,	type = "bs",	offsetx = 0,	offsety = -70,	texturepath = "Interface\\Icons\\inv_ingot_thorium",	spell = 46353},
	profbars_bsbutton20 = {item=36916,	type = "bs",	offsetx = 35,	offsety = -70,	texturepath = "Interface\\Icons\\inv_ingot_cobalt",	spell = 49252},
	profbars_bsbutton21 = {item=36913,	type = "bs",	offsetx = 70,	offsety = -70,	texturepath = "Interface\\Icons\\inv_ingot_yoggthorite",	spell = 49258},
	profbars_bsbutton22 = {item=41163,	type = "bs",	offsetx = 105,	offsety = -70,	texturepath = "Interface\\Icons\\inv_ingot_platinum",	spell = 55211},
	profbars_bsbutton23 = {item=37663,	type = "bs",	offsetx = 140,	offsety = -70,	texturepath = "Interface\\Icons\\inv_ingot_titansteel_blue",	spell = 55208},
}

------------------------------------------------------------------------------------------------------
-- Table of Item Levels and associated Enchanting Skills
------------------------------------------------------------------------------------------------------
local green_disenchant = {
	[1] = {skill = 1,		minlvl = 0,		maxlvl = 20},
	[2] = {skill = 25,	minlvl = 21,	maxlvl = 25},
	[3] = {skill = 50,	minlvl = 26,	maxlvl = 30},
	[4] = {skill = 75,	minlvl = 31,	maxlvl = 35},
	[5] = {skill = 100,	minlvl = 36,	maxlvl = 40},
	[6] = {skill = 125,	minlvl = 41,	maxlvl = 45},
	[7] = {skill = 150,	minlvl = 46,	maxlvl = 50},
	[8] = {skill = 175,	minlvl = 51,	maxlvl = 55},
	[9] = {skill = 200,	minlvl = 56,	maxlvl = 60},
	[10] = {skill = 225,	minlvl = 61,	maxlvl = 99},
	[11] = {skill = 275,	minlvl = 100,	maxlvl = 120},
	[12] = {skill = 325,	minlvl = 130,	maxlvl = 153},
	[13] = {skill = 350,	minlvl = 154,	maxlvl = 999}
}
local blue_disenchant = {
	[1] = {skill = 1,		minlvl = 0,		maxlvl = 20},
	[2] = {skill = 25,	minlvl = 21,	maxlvl = 25},
	[3] = {skill = 50,	minlvl = 26,	maxlvl = 30},
	[4] = {skill = 75,	minlvl = 31,	maxlvl = 35},
	[5] = {skill = 100,	minlvl = 36,	maxlvl = 40},
	[6] = {skill = 125,	minlvl = 41,	maxlvl = 45},
	[7] = {skill = 150,	minlvl = 46,	maxlvl = 50},
	[8] = {skill = 175,	minlvl = 51,	maxlvl = 55},
	[9] = {skill = 200,	minlvl = 56,	maxlvl = 60},
	[10] = {skill = 225,	minlvl = 61,	maxlvl = 99},
	[11] = {skill = 275,	minlvl = 100,	maxlvl = 120},
	[12] = {skill = 325,	minlvl = 130,	maxlvl = 999},
}
local purple_disenchant = {
	[1] = {skill = 1,		minlvl = 0,		maxlvl = 20},
	[2] = {skill = 25,	minlvl = 21,	maxlvl = 25},
	[3] = {skill = 50,	minlvl = 26,	maxlvl = 30},
	[4] = {skill = 75,	minlvl = 31,	maxlvl = 35},
	[5] = {skill = 100,	minlvl = 36,	maxlvl = 40},
	[6] = {skill = 125,	minlvl = 41,	maxlvl = 45},
	[7] = {skill = 150,	minlvl = 46,	maxlvl = 50},
	[8] = {skill = 175,	minlvl = 51,	maxlvl = 55},
	[9] = {skill = 200,	minlvl = 56,	maxlvl = 60},
	[10] = {skill = 225,	minlvl = 61,	maxlvl = 92},
	[11] = {skill = 300,	minlvl = 93,	maxlvl = 164},
	[12] = {skill = 375,	minlvl = 165,	maxlvl = 999},
}

------------------------------------------------------------------------------------------------------
-- Open the config dialog
------------------------------------------------------------------------------------------------------
function ProfessionBars:OpenConfig()
	InterfaceOptionsFrame_OpenToCategory("ProfessionBars")
end

------------------------------------------------------------------------------------------------------
-- Gets a player's level in a skill
------------------------------------------------------------------------------------------------------
function ProfessionBars:GetLevel(id)
	local spell = GetSpellInfo(id)
	for i=1, GetNumSkillLines() do
		local name, _, _, level = GetSkillLineInfo(i);
		if name == spell then
			return level
		end
	end
end

------------------------------------------------------------------------------------------------------
-- Searches a player's bags for green or higher items and returns tables containing those
------------------------------------------------------------------------------------------------------
function ProfessionBars:GetDisenchantableItems()
	local green = {};
	local blue = {};
	local purple = {};
	local bag, slot;
	for bag=0, 4 do
		local slots = GetContainerNumSlots(bag);
		for slot=1, slots do
			local name, rarity, texture, type, equipslot, ilvl
			local link = GetContainerItemLink(bag, slot)
			if link then
				name, _, rarity, ilvl, _, type, _, _, equipslot = GetItemInfo(link);
				texture = GetContainerItemInfo(bag, slot);
				local itemid = string.match(link, "^|%x+|Hitem:(%d+):");
				if (type == "Armor" or type == "Weapon") and equipslot ~= "INVTYPE_BODY" and equipslot ~="INVTYPE_TABARD" and (not db.blacklist[itemid])  then
					local level = ProfessionBars:GetLevel(7411);
					local requiredlevel
					if rarity == 2 then
						for disenchanttableentry=1, #green_disenchant do
							if green_disenchant[disenchanttableentry]["minlvl"] <= ilvl and green_disenchant[disenchanttableentry]["maxlvl"] >= ilvl then
								requiredlevel = green_disenchant[disenchanttableentry]["skill"];
								break;
							end
						end
						if level >= requiredlevel then
							local entry = #green+1;
							green[entry] = {};
							green[entry]["bag"] = bag;
							green[entry]["slot"] = slot;
							green[entry]["name"] = name;
							green[entry]["texture"] = texture;
							green[entry]["link"] = link;
						end
					elseif rarity == 3 then
						for disenchanttableentry=1, #blue_disenchant do
							if blue_disenchant[disenchanttableentry]["minlvl"] <= ilvl and blue_disenchant[disenchanttableentry]["maxlvl"] >= ilvl then
								requiredlevel = blue_disenchant[disenchanttableentry]["skill"];
								break;
							end
						end
						if level >= requiredlevel then
							local entry = #blue+1;
							blue[entry] = {};
							blue[entry]["bag"] = bag;
							blue[entry]["slot"] = slot;
							blue[entry]["name"] = name;
							blue[entry]["texture"] = texture;
							blue[entry]["link"] = link;
						end
					elseif rarity == 4 then
						for disenchanttableentry=1, #purple_disenchant do
							if purple_disenchant[disenchanttableentry]["minlvl"] <= ilvl and purple_disenchant[disenchanttableentry]["maxlvl"] >= ilvl then
								requiredlevel = purple_disenchant[disenchanttableentry]["skill"];
								break;
							end
						end
						if level >= requiredlevel then
							local entry = #purple+1;
							purple[entry] = {};
							purple[entry]["bag"] = bag;
							purple[entry]["slot"] = slot;
							purple[entry]["name"] = name;
							purple[entry]["texture"] = texture;
							purple[entry]["link"] = link;
						end
					end
				end
			end
		end
	end
	return green, blue, purple
end

------------------------------------------------------------------------------------------------------
-- Searches a player's bags for an item id
------------------------------------------------------------------------------------------------------
local function SearchPlayerBags(itemID, occurance)
	--print("-------------");
	local found = 0;
	for i=0, 4 do
		local a = GetContainerNumSlots(i)
		for c=1, a do
			local link = GetContainerItemLink(i, c);
			local name = GetItemInfo(itemID);
			if link then
				if string.match(link, name) then
					found = found+1;
					--print("Occurance "..found..": Bag "..i..", Slot "..c..": Item is "..link..".");
					if found == occurance then
						--print("-------------");
						return i, c
					end
				end
			end
		end
	end
	--print("-------------");
	return nil
end

------------------------------------------------------------------------------------------------------
-- Add the help text to the various buttons
------------------------------------------------------------------------------------------------------
local function AddHelp(self, type)
	if not db.help then
		return;
	end

	self:AddLine(" ");
	if type == "jc" then
		self:AddLine("Left click to prospect the highest level ore");
		self:AddLine("Right click to prospect the lowest level ore");
	elseif type == "insc" then
		self:AddLine("Left click to mill the highest level herbs");
		self:AddLine("Right click to mill the lowest level herbs");
	elseif type == "lw" then
		self:AddLine("Left click to upgrade as much as possible");
		self:AddLine("Right click to upgrade once");
	elseif type == "bs" then
		self:AddLine("Click to smelt one bar");
	end
end

------------------------------------------------------------------------------------------------------
-- What happens when the tooltip is shown
------------------------------------------------------------------------------------------------------
local function OnTooltipShow(self, frame)
	local a = frame:GetName();
	if string.match(a, "profbars_enchbutton") then
		local id = tonumber((string.gsub(a, "profbars_enchbutton", "")));
		self:SetHyperlink(profbars_temp["enchantingindex"][id]["link"]);
		return
	end
	if not profbars_frames[a] then
		-- Frame didn't exist
		return
	end
	-- Add the normal item tooltip
	self:SetHyperlink("item:"..profbars_frames[a]["item"]..":0:0:0:0:0:0:0");
	
	-- Add the help text (won't add anything if the option says not to)
	AddHelp(self, profbars_frames[a]["type"]);
end

------------------------------------------------------------------------------------------------------
-- What happens when we mouseover a button
------------------------------------------------------------------------------------------------------
local function OnEnter(self)
	GameTooltip:SetOwner(self, "ANCHOR_NONE");
	GameTooltip:ClearAllPoints();
	GameTooltip:SetPoint("TOPLEFT", self, "BOTTOMLEFT");
	GameTooltip:ClearLines()
	OnTooltipShow(GameTooltip, self)
	GameTooltip:Show()
end

------------------------------------------------------------------------------------------------------
-- What happens when the mouse leaves the button
------------------------------------------------------------------------------------------------------
local function OnLeave(self)
	GameTooltip:Hide();
end

------------------------------------------------------------------------------------------------------
-- Check if a player has a specific spell
------------------------------------------------------------------------------------------------------
local function PlayerHasSpell(spellID)
	-- Check if we have this in our spellbook
	return GetSpellInfo((GetSpellInfo(spellID)));
end

------------------------------------------------------------------------------------------------------
-- What happens when the "bar mode" changes
------------------------------------------------------------------------------------------------------
function ProfessionBars:ModeChange()
	-- Do any potential button changes
	ProfessionBars:ITEM_PUSH();

	if db.mode == 1 then
		-- We're now dislocating from tradeskill frame

		if PlayerHasSpell(25229) then
			-- Jewelcrafting
			ProfessionBars:ExecuteFrameAction(profbars_jewelcrafting, "ClearAllPoints")
			ProfessionBars:ExecuteFrameAction(profbars_jewelcrafting, "SetPoint", {"BOTTOMLEFT", UIParent, db.offsets.jc.x, db.offsets.jc.y})
			ProfessionBars:ExecuteFrameAction(profbars_jcmove, "Show")
			ProfessionBars:ExecuteFrameAction(profbars_jewelcrafting, "Show")
		end

		if PlayerHasSpell(2108) then
			--Leatherworking
			ProfessionBars:ExecuteFrameAction(profbars_leatherworking, "ClearAllPoints")
			ProfessionBars:ExecuteFrameAction(profbars_leatherworking, "SetPoint", {"BOTTOMLEFT", UIParent, db.offsets.lw.x, db.offsets.lw.y})
			ProfessionBars:ExecuteFrameAction(profbars_lwmove, "Show")
			ProfessionBars:ExecuteFrameAction(profbars_leatherworking, "Show")
		end
		
		if PlayerHasSpell(2259) then
			-- Alchemy
		end

		if PlayerHasSpell(2018) and PlayerHasSpell(2656) then
			-- Blacksmithing (and Mining)
			ProfessionBars:ExecuteFrameAction(profbars_blacksmithing, "ClearAllPoints")
			ProfessionBars:ExecuteFrameAction(profbars_blacksmithing, "SetPoint", {"BOTTOMLEFT", UIParent, db.offsets.bs.x, db.offsets.bs.y})
			ProfessionBars:ExecuteFrameAction(profbars_bsmove, "Show")
			ProfessionBars:ExecuteFrameAction(profbars_blacksmithing, "Show")
		end

		if PlayerHasSpell(7411) then
			-- Enchanting
			ProfessionBars:ExecuteFrameAction(profbars_enchanting, "ClearAllPoints")
			ProfessionBars:ExecuteFrameAction(profbars_enchanting, "SetPoint", {"BOTTOMLEFT", UIParent, db.offsets.ench.x, db.offsets.ench.y})
			ProfessionBars:ExecuteFrameAction(profbars_enchmove, "Show")
			ProfessionBars:ExecuteFrameAction(profbars_enchanting, "Show")
		end

		if PlayerHasSpell(4036) then	
			-- Engineering
		end

		if PlayerHasSpell(2366) then
			-- Herbalism
		end

		if PlayerHasSpell(45357) then
			-- Inscription
			ProfessionBars:ExecuteFrameAction(profbars_inscription, "ClearAllPoints")
			ProfessionBars:ExecuteFrameAction(profbars_inscription, "SetPoint", {"BOTTOMLEFT", UIParent, db.offsets.insc.x, db.offsets.insc.y})
			ProfessionBars:ExecuteFrameAction(profbars_inscmove, "Show")
			ProfessionBars:ExecuteFrameAction(profbars_inscription, "Show")
		end
	else
		-- We're now attaching to tradeskill frame

		if PlayerHasSpell(25229) then
			ProfessionBars:ExecuteFrameAction(profbars_jcmove, "Hide")
			ProfessionBars:ExecuteFrameAction(profbars_jewelcrafting, "Hide")
			ProfessionBars:ExecuteFrameAction(profbars_jewelcrafting, "SetWidth", function() return profbars_jewelcrafting:GetWidth() - 17.5 end)
		end

		if PlayerHasSpell(45357) then
			ProfessionBars:ExecuteFrameAction(profbars_inscmove, "Hide")
			ProfessionBars:ExecuteFrameAction(profbars_inscription, "Hide")
			ProfessionBars:ExecuteFrameAction(profbars_inscription, "SetWidth", function() return profbars_inscription:GetWidth() - 17.5 end)
		end
		
		if PlayerHasSpell(2108) then
			ProfessionBars:ExecuteFrameAction(profbars_lwmove, "Hide")
			ProfessionBars:ExecuteFrameAction(profbars_leatherworking, "Hide")
			ProfessionBars:ExecuteFrameAction(profbars_leatherworking, "Hide", function() return profbars_leatherworking:GetWidth() - 17.5 end)
		end
		
		if PlayerHasSpell(7411) then
			ProfessionBars:ExecuteFrameAction(profbars_enchmove, "Hide")
			ProfessionBars:ExecuteFrameAction(profbars_enchanting, "Hide")
			ProfessionBars:ExecuteFrameAction(profbars_enchanting, "SetWidth", function() return profbars_enchanting:GetWidth() - 17.5 end)
		end
		
		if PlayerHasSpell(2018) and PlayerHasSpell(2656) then
			ProfessionBars:ExecuteFrameAction(profbars_bsmove, "Hide")
			ProfessionBars:ExecuteFrameAction(profbars_blacksmithing, "Hide")
			ProfessionBars:ExecuteFrameAction(profbars_blackstithing, "SetWidth", function() return profbars_blacksmithing:GetWidth() - 17.5 end)
		end
	end
end

------------------------------------------------------------------------------------------------------
-- Function to grab the number of stacks of a certain item in our bags
------------------------------------------------------------------------------------------------------
local function GetNumStacks(itemID)
	local found = 0;
	for i = 0, 4 do
		local a =GetContainerNumSlots(i)
		for c = 1, a do
			local link = GetContainerItemLink(i, c);
			local name = GetItemInfo(itemID);
			if link and string.match(link, name) then
				found = found+1;
			end
		end
	end
	return found
end

------------------------------------------------------------------------------------------------------
-- Wrapper function for our options table
------------------------------------------------------------------------------------------------------
local function giveOptions()
	local options = {
		type = "group",
		name = "ProfessionBars",
		get = function(k) return db[k.arg] end,
		set = function(k, v) db[k.arg] = v; end,
		args = {
			mode = {
				name = "Mode",
				type = "select",
				width = "double",
				values = {
					[1] = "Separate (draggable) bar",
					[2] = "Attached to Tradeskill window",
				},
				arg = "mode",
				set = function(k, v) db.mode = v; ProfessionBars:ModeChange() end,
			},
			help = {
				type = "toggle",
				name = "Show Help Text",
				desc = "Turns help On/Off",
				arg = "help",
			},
		},
	}
	return options;
end

------------------------------------------------------------------------------------------------------
-- What happens when the addon is first initialised
------------------------------------------------------------------------------------------------------
function ProfessionBars:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("ProfBarsDB", defaults, "Default")
	db = self.db.profile

	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("ProfessionBars", giveOptions)
	local frame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("ProfessionBars", "ProfessionBars")
	
	self:RegisterChatCommand("profbars",		"OpenConfig");
	self:RegisterChatCommand("professionbars",	"OpenConfig");
	LibStub("AceConfig-3.0"):RegisterOptionsTable("ProfessionBarsBlacklist", profbars_debugoptions, {"profblacklist", "pbblacklist"});
end

------------------------------------------------------------------------------------------------------
-- What happens when the addon is enabled
------------------------------------------------------------------------------------------------------
function ProfessionBars:OnEnable()
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	self:RegisterEvent("CHAT_MSG_SKILL");
	self:RegisterEvent("TRADE_SKILL_SHOW");
	self:RegisterEvent("TRADE_SKILL_CLOSE");
	self:RegisterEvent("ITEM_PUSH");
	self:RegisterEvent("UNIT_SPELLCAST_FAILED");
	self:RegisterEvent("PLAYER_REGEN_ENABLED", "DoScheduledActions")
end

------------------------------------------------------------------------------------------------------
-- Craft a Leatherworking Recipe
------------------------------------------------------------------------------------------------------

function ProfessionBars:DoLwRecipe(button, name)
	if db.mode == 1 then
		CastSpellByName((GetSpellInfo(2108)));
	end
	local itemname = GetItemInfo(profbars_frames[name]["item"]);
	local index, numavailable
	for i=1, GetNumTradeSkills() do
		tempname,_,tempavailable = GetTradeSkillInfo(i);
		if tempname == itemname then
			index = i;
			numavailable = tempavailable;
			break;
		end
	end
	if index then
		if button == "RightButton" then
			DoTradeSkill(index, 1);
		else
			DoTradeSkill(index, numavailable);
		end
	else
		print("You don't seem to have the required recipe.");
	end
	if db.mode == 1 then
		CloseTradeSkill();
	end
end

------------------------------------------------------------------------------------------------------
-- Do a Smelting Recipe
------------------------------------------------------------------------------------------------------

function ProfessionBars:DoMiningRecipe(button, name)
	CastSpellByName((GetSpellInfo(2656)));
	local spellname = GetSpellInfo(profbars_frames[name]["spell"]);
	local index, numavailable
	for i=1, GetNumTradeSkills() do
		tempname,_,tempavailable = GetTradeSkillInfo(i);
		if tempname == spellname then
			index = i;
			numavailable = tempavailable;
			break;
		end
	end
	if index then
		DoTradeSkill(index, 1);
	else
		print("You don't seem to have the required recipe.");
	end
	CloseTradeSkill();
	if db.mode == 2 then
		CastSpellByName((GetSpellInfo(2018)));
	end
end

------------------------------------------------------------------------------------------------------
-- The delay for filtering disenchantable items ends
------------------------------------------------------------------------------------------------------
function ProfessionBars:NoLongerNotDisenchantable()
	profbars_temp["lastdisenchant"] = nil;
	self:UnregisterEvent("UNIT_SPELLCAST_FAILED");
end

------------------------------------------------------------------------------------------------------
-- Add an item to the blacklist
------------------------------------------------------------------------------------------------------
function ProfessionBars:AddBlacklist(info, input)
	local itemid = string.match(input, "^|%x+|Hitem:(%d+):");
	db.blacklist[itemid] = true;
	ProfessionBars:ITEM_PUSH();
end


------------------------------------------------------------------------------------------------------
-- Remove an item from the blacklist
------------------------------------------------------------------------------------------------------
function ProfessionBars:RemoveBlacklist(info, input)
	local itemid = string.match(input, "^|%x+|Hitem:(%d+):");
	db.blacklist[itemid] = false;
	ProfessionBars:ITEM_PUSH();
end
	

------------------------------------------------------------------------------------------------------
-- A spellcast failed during the nondisenchantable delay.
------------------------------------------------------------------------------------------------------
function ProfessionBars:UNIT_SPELLCAST_FAILED(event, source, spellname)
	if source == "player" and spellname == GetSpellInfo(13262) then
		if profbars_temp["lastdisenchant"] then
			db.blacklist[profbars_temp["lastdisenchant"]] = true;
			ProfessionBars:ITEM_PUSH();
		end
	end
end

------------------------------------------------------------------------------------------------------
-- What happens when a button is clicked
------------------------------------------------------------------------------------------------------
local function ButtonClick(self, button, down)
	-- Fetch the name of the button
	local name = self:GetName();
	if string.match(name, "profbars_enchbutton") then
		--Enchanting
		local id = tonumber((string.gsub(name, "profbars_enchbutton", "")));
		local itemid = string.match(profbars_temp["enchantingindex"][id]["link"], "^|%x+|Hitem:(%d+):");
		profbars_temp["lastdisenchant"] = itemid;
		ProfessionBars:ScheduleTimer("NoLongerNotDisenchantable", 2);
		self:RegisterEvent("UNIT_SPELLCAST_FAILED");
		self:SetAttribute("type", "spell");
		self:SetAttribute("spell", GetSpellInfo(13262));
		self:SetAttribute("target-bag", profbars_temp["enchantingindex"][id]["bag"]);
		self:SetAttribute("target-slot", profbars_temp["enchantingindex"][id]["slot"]);
		ProfessionBars:ScheduleTimer("ITEM_PUSH", 5);
		return
	end
	-- If we are doing something with leatherworking, we don't need the other locals
	if profbars_frames[name]["type"] == "lw" then
		ProfessionBars:DoLwRecipe(button, name);
		return
	-- Same goes for blacksmithing
	elseif profbars_frames[name]["type"] == "bs" then
		ProfessionBars:DoMiningRecipe(button, name);
		return
	end
	
	-- Store the spell we will be using
	local spell;
	
	-- Store item we will be using
	local item;

	-- Table for the used herbs
	local herbs;

	if (name == "profbars_inscbutton9") then
		-- Milling

		herbs = {765, 2447, 2449};
	elseif (name == "profbars_inscbutton8") then
		-- Milling

		if (not (button == "RightButton")) and (ProfessionBars:GetLevel(45357) < 25) and (GetItemCount(785) >= 5) then
			spell = 51005;
			item = 785;
		else
			herbs = {785, 2452, 2450, 3820, 2453};
		end

	elseif (name == "profbars_inscbutton7") then
		herbs = {3355, 3369, 3356, 3357};

	elseif (name == "profbars_inscbutton6") then
		herbs = {3818, 3821, 3358, 3819};

	elseif (name == "profbars_inscbutton5") then
		herbs = {4625, 8831, 8836, 8838, 8839, 8846};
		
	elseif (name == "profbars_inscbutton4") then
		herbs = {13464, 13463, 13465, 13466, 13467};
	
	-- elseif (name == "profbars_inscbutton3") then
		-- Doesn't exist
		-- spell = 51005;


		
	elseif (name == "profbars_inscbutton2") then
		herbs = {36904, 37921, 36901, 36907, 39970, 39969, 36902, 36903, 36906, 36905};
		
	elseif (name == "profbars_inscbutton1") then
		herbs = {22789, 22786, 22785, 22787, 22790, 22791, 22793};

	elseif profbars_frames[name]["type"] == "jc" then
		spell = 31252;
		item = profbars_frames[name]["item"];
	end
	if herbs then
		spell = 51005;
		if button == "RightButton" then
			for i=1, #herbs do
				if GetItemCount(herbs[i]) >= 5 then
					item = herbs[i];
					break;
				end
			end
		else
			i = #herbs;
			while i >= 1 do
				if GetItemCount(herbs[i]) >= 5 then
					item = herbs[i];
					break;
				end
				i = i-1;
			end
		end
	end
	if not item then
		-- We lacked source material
		if spell == 31252 then
			--Jewelcrafting
			print("You don't have enough ore to prospect.");
		elseif spell == 51005 then
			--Inscription
			print("You don't have enough herbs to mill.");
		end
		return;
	end
	
	-- Important variables
	local a, b = SearchPlayerBags(item, 1);
	local _, c = GetContainerItemInfo(a, b);
	local d, e, f

	if c < 5 then
		d, e = SearchPlayerBags(item, 2);
		_, f = GetContainerItemInfo(d, e);
		if (c + f) < 5 then
			SplitContainerItem(d, e, f);
		else
			SplitContainerItem(d, e, (5 - c));
		end
		PickupContainerItem(a, b);
		print("Re-stacked items, click again to start.");
	else
		self:SetAttribute("type", "spell");
		self:SetAttribute("spell", GetSpellInfo(spell));
		self:SetAttribute("target-item", GetItemInfo(item));
	end
end

------------------------------------------------------------------------------------------------------
-- Function for creating a new button
------------------------------------------------------------------------------------------------------
local function CreateButton(name, parent)
	-- Create the button frame
	local btn = CreateFrame("Button", name, parent, "SecureActionButtonTemplate");
	
	-- Set height
	ProfessionBars:ExecuteFrameAction(btn, "SetHeight", 35)

	-- Set width
	ProfessionBars:ExecuteFrameAction(btn, "SetWidth", 35)

	-- Enable it for clicks
	ProfessionBars:ExecuteFrameAction(btn, "SetScript", {"PreClick", ButtonClick})
	ProfessionBars:ExecuteFrameAction(btn, "RegisterForClicks", "AnyUp")

	-- Position it
	ProfessionBars:ExecuteFrameAction(btn, "SetPoint", {"TOPLEFT", parent, "TOPLEFT", profbars_frames[name].offsetx, profbars_frames[name].offsety});

	-- Set textures
	ProfessionBars:ExecuteFrameAction(btn, "SetNormalTexture", profbars_frames[name].texturepath);
	ProfessionBars:ExecuteFrameAction(btn, "SetHighlightTexture", {profbars_frames[name].texturepath, 1.5});

	-- Set mouseover/mouseout
	ProfessionBars:ExecuteFrameAction(btn, "SetScript", {"OnEnter", OnEnter})
	ProfessionBars:ExecuteFrameAction(btn, "SetScript", {"OnLeave", OnLeave})
end

------------------------------------------------------------------------------------------------------
-- Function that creates all the buttons that are needed
------------------------------------------------------------------------------------------------------
function ProfessionBars:PLAYER_ENTERING_WORLD()

	-- Check if we even have a profession
	local haveProfession = false;

	if PlayerHasSpell(2259) then
		-- Alchemy
		
		-- We have a profession
		--haveProfession = true;
	end
	if PlayerHasSpell(2018) then
		-- Blacksmithing
		if not profbars_temp.buttonscreated then
			--Create the main frame
			bs_bar = CreateFrame("Frame", "profbars_blacksmithing", UIParent);
			--Create the move button for the Blacksmithing frame
			bs_movebutton = CreateFrame("Button", "profbars_bsmove", profbars_blacksmithing);
		end
		ProfessionBars:ExecuteFrameAction(bs_bar, "SetHeight", 0)
		ProfessionBars:ExecuteFrameAction(bs_bar, "SetMovable", true)
		ProfessionBars:ExecuteFrameAction(bs_bar, "SetPoint", {"BOTTOMLEFT", "UIParent", db.offsets.bs.x, db.offsets.bs.y})
		ProfessionBars:ExecuteFrameAction(bs_bar, "SetWidth", 297.5)
		ProfessionBars:ExecuteFrameAction(bs_movebutton, "SetHeight", 17.5)
		ProfessionBars:ExecuteFrameAction(bs_movebutton, "SetWidth", 17.5)
		ProfessionBars:ExecuteFrameAction(bs_movebutton, "SetScript", {"OnMouseDown", profbars_startdragging_bs})
		ProfessionBars:ExecuteFrameAction(bs_movebutton, "SetScript", {"OnMouseUp", profbars_stopdragging_bs})
		ProfessionBars:ExecuteFrameAction(bs_movebutton, "SetPoint", {"TOPRIGHT", profbars_blacksmithing, "TOPRIGHT"})
		ProfessionBars:ExecuteFrameAction(bs_movebutton, "SetHighlightTexture", {"Interface\\Icons\\Spell_Arcane_PortalStormWind", 0.8});
		
		for i=1, 23 do
			CreateButton("profbars_bsbutton"..i, profbars_blacksmithing);
		end
		if db.mode == 2 then
			ProfessionBars:ExecuteFrameAction(bs_bar, "ClearAllPoints")
			if IsAddOnLoaded("AdvancedTradeSkillWindow") then
				ProfessionBars:ExecuteFrameAction(bs_bar, "SetPoint", {"TOP", ATSWFrame, "BOTTOM", 0, 10})
			elseif IsAddOnLoaded("GnomeWorks") then
				ProfessionBars:ExecuteFrameAction(bs_bar, "SetPoint", {"TOP", GnomeWorksFrame, "BOTTOM", 0, -5})
			else
				ProfessionBars:ExecuteFrameAction(bs_bar, "SetPoint", {"TOP", TradeSkillFrame, "BOTTOMLEFT", 180, 75});
			end
			ProfessionBars:ExecuteFrameAction(bs_movebutton, "Hide")
			ProfessionBars:ExecuteFrameAction(bs_bar, "Hide")
		end
		-- We have a profession
		haveProfession = true;
	end

	if PlayerHasSpell(7411) then
		-- Enchanting
		if not profbars_temp.buttonscreated then
			--Create the main frame
			ench_bar = CreateFrame("Frame", "profbars_enchanting", UIParent);
			--Create the move button for the Enchanting frame
			ench_movebutton = CreateFrame("Button", "profbars_enchmove", profbars_enchanting);
		end
		-- Set height
		ProfessionBars:ExecuteFrameAction(ench_bar, "SetHeight", 0)
		-- Set movable
		ProfessionBars:ExecuteFrameAction(ench_bar, "SetMovable", true)
		-- Parent it to UIParent and where it was saved
		ProfessionBars:ExecuteFrameAction(ench_bar, "SetPoint", {"BOTTOMLEFT", UIParent, db.offsets.ench.x, db.offsets.ench.y})
		-- Set default width
		ProfessionBars:ExecuteFrameAction(ench_bar, "SetWidth", 332.5)
		ProfessionBars:ExecuteFrameAction(ench_movebutton, "SetHeight", 17.5)
		ProfessionBars:ExecuteFrameAction(ench_movebutton, "SetWidth", 17.5)
		ProfessionBars:ExecuteFrameAction(ench_movebutton, "SetScript", {"OnMouseDown", profbars_startdragging_ench})
		ProfessionBars:ExecuteFrameAction(ench_movebutton, "SetScript", {"OnMouseUp", profbars_stopdragging_ench})
		ProfessionBars:ExecuteFrameAction(ench_movebutton, "SetPoint", {"TOPRIGHT", profbars_enchanting, "TOPRIGHT"})
		ProfessionBars:ExecuteFrameAction(ench_movebutton, "SetHighlightTexture", {"Interface\\Icons\\Spell_Arcane_PortanStormWind", 0.8})
		if db.mode == 2 then
			-- We're attaching to Tradeskill window
			ProfessionBars:ExecuteFrameAction(ench_bar, "ClearAllPoints")
			if IsAddOnLoaded("AdvancedTradeSkillWindow") then
				ProfessionBars:ExecuteFrameAction(ench_bar, "SetPoint", {"TOP", ATSWFrame, "BOTTOM", 0, 10})
			elseif IsAddOnLoaded("GnomeWorks") then
				ProfessionBars:ExecuteFrameAction(ench_bar, "SetPoint", {"TOP", GnomeWorksFrame, "BOTTOM", 0, -5})
			else
				ProfessionBars:ExecuteFrameAction(ench_bar, "SetPoint", {"TOP", TradeSkillFrame, "BOTTOMLEFT", 180, 75})
			end

			-- We don't need the move button, or the main bar window
			ProfessionBars:ExecuteFrameAction(ench_movebutton, "Hide")
			ProfessionBars:ExecuteFrameAction(ench_bar, "Hide")
		end
		-- We have a profession
		haveProfession = true;
	end

	if PlayerHasSpell(4036) then	
		-- Engineering
		
		-- We have a profession
		--haveProfession = true;
	end

	if PlayerHasSpell(2366) then
		-- Herbalism
		
		-- We have a profession
		--haveProfession = true;
	end

	if PlayerHasSpell(45357) then
		-- Inscription
		if not profbars_temp.buttonscreated then
			-- Create the main frame
			insc_bar = CreateFrame("Frame", "profbars_inscription", UIParent);
			-- Create the Move button for the Inscription frame
			insc_movebutton = CreateFrame("Button", "profbars_inscmove", profbars_inscription);
		end
		-- Set height
		ProfessionBars:ExecuteFrameAction(insc_bar, "SetHeight", 35)

		-- Set movable
		ProfessionBars:ExecuteFrameAction(insc_bar, "SetMovable", true)
		
		-- Parent it to UIParent and position where we saved it
		ProfessionBars:ExecuteFrameAction(insc_bar, "SetPoint", {"BOTTOMLEFT", UIParent, db.offsets.insc.x, db.offsets.insc.y})
		
		-- Set default width
		ProfessionBars:ExecuteFrameAction(insc_bar, "SetWidth", 297.5)

		ProfessionBars:ExecuteFrameAction(insc_movebutton, "SetHeight", 17.5)
		ProfessionBars:ExecuteFrameAction(insc_movebutton, "SetWidth", 17.5)
		ProfessionBars:ExecuteFrameAction(insc_movebutton, "SetScript", {"OnMouseDown", profbars_startdragging_insc})
		ProfessionBars:ExecuteFrameAction(insc_movebutton, "SetScript", {"OnMouseUp", profbars_stopdragging_insc})
		ProfessionBars:ExecuteFrameAction(insc_movebutton, "SetPoint", {"TOPRIGHT", profbars_inscription, "TOPRIGHT"})
		ProfessionBars:ExecuteFrameAction(insc_movebutton, "SetHighlightTexture", {"Interface\\Icons\\Spell_Arcane_PortalStormWind", 0.8})
		
		for i=1, 9 do
			if i~=3 then
				CreateButton("profbars_inscbutton"..i, profbars_inscription);
			end
		end
		
		if db.mode == 2 then
			-- We're attaching to Tradeskill window
			ProfessionBars:ExecuteFrameAction(insc_bar, "ClearAllPoints")
			if IsAddOnLoaded("AdvancedTradeSkillWindow") then
				ProfessionBars:ExecuteFrameAction(insc_bar, "SetPoint", {"TOP", ATSWFrame, "BOTTOM", 0, 10})
			elseif IsAddOnLoaded("GnomeWorks") then
				ProfessionBars:ExecuteFrameAction(insc_bar, "SetPoint", {"TOP", GnomeWorksFrame, "BOTTOM", 0, -5})
			else
				ProfessionBars:ExecuteFrameAction(insc_bar, "SetPoint", {"TOP", TradeSkillFrame, "BOTTOMLEFT", 180, 75})
			end

			-- We don't need the move button, or the main bar window
			ProfessionBars:ExecuteFrameAction(insc_movebutton, "Hide")
			ProfessionBars:ExecuteFrameAction(insc_bar, "Hide")
		end
		
		-- We have a profession
		haveProfession = true;
	end

	if PlayerHasSpell(25229) then
		-- Jewelcrafting
		if not profbars_temp.buttonscreated then
			-- Create the main frame
			jc_bar = CreateFrame("Frame", "profbars_jewelcrafting", UIParent);
			-- Create the Move button for the JC frame
			jc_movebutton = CreateFrame("Button", "profbars_jcmove", profbars_jewelcrafting);
		end
		-- Set height
		ProfessionBars:ExecuteFrameAction(jc_bar, "SetHeight", 35)
		
		-- Set movable
		ProfessionBars:ExecuteFrameAction(jc_bar, "SetMovable", true)
		
		-- Parent it to UIParent and position where we saved it
		ProfessionBars:ExecuteFrameAction(jc_bar, "SetPoint", {"BOTTOMLEFT", UIParent, db.offsets.jc.x, db.offsets.jc.y})
		
		-- Set default width
		ProfessionBars:ExecuteFrameAction(jc_bar, "SetWidth", 332.5)
		ProfessionBars:ExecuteFrameAction(jc_movebutton, "SetHeight", 17.5)
		ProfessionBars:ExecuteFrameAction(jc_movebutton, "SetWidth", 17.5)
		ProfessionBars:ExecuteFrameAction(jc_movebutton, "SetScript", {"OnMouseDown", profbars_startdragging_jc})
		ProfessionBars:ExecuteFrameAction(jc_movebutton, "SetScript", {"OnMouseUp", profbars_stopdragging_jc})
		ProfessionBars:ExecuteFrameAction(jc_movebutton, "SetPoint", {"TOPRIGHT", profbars_jewelcrafting, "TOPRIGHT"})
		ProfessionBars:ExecuteFrameAction(jc_movebutton, "SetHighlightTexture", {"Interface\\Icons\\Spell_Arcane_PortalStormWind", 0.8})
		if not profbars_temp.buttonscreated then
			for i=1, 10 do
				CreateButton("profbars_jcbutton"..i, profbars_jewelcrafting)
			end
		end

		if db.mode == 2 then
			-- We're attaching to Tradeskill window
			ProfessionBars:ExecuteFrameAction(jc_bar, "ClearAllPoints")
			if IsAddOnLoaded("AdvancedTradeSkillWindow") then
				ProfessionBars:ExecuteFrameAction(jc_bar, "SetPoint", {"TOP", ATSWFrame, "BOTTOM", 0, 10})
			elseif IsAddOnLoaded("GnomeWorks") then
				ProfessionBars:ExecuteFrameAction(jc_bar, "SetPoint", {"TOP", GnomeWorksFrame, "BOTTOM", 0, -5})
			else
				ProfessionBars:ExecuteFrameAction(jc_bar, "SetPoint", {"TOP", TradeSkillFrame, "BOTTOMLEFT", 180, 75})
			end
			
			-- We don't need the move button, or the main bar window
			ProfessionBars:ExecuteFrameAction(jc_movebutton, "Hide")
			ProfessionBars:ExecuteFrameAction(jc_bar, "Hide")
		end
		
		-- We have a profession
		haveProfession = true;
	end

	if PlayerHasSpell(2108) then
		-- Leatherworking
		if not profbars_temp.buttonscreated then
			lw_bar = CreateFrame("Frame", "profbars_leatherworking", UIParent);
			lw_movebutton = CreateFrame("Button", "profbars_lwmove", profbars_jewelcrafting);
		end
		ProfessionBars:ExecuteFrameAction(lw_bar, "SetHeight", 70)
		ProfessionBars:ExecuteFrameAction(lw_bar, "SetMovable", true)
		ProfessionBars:ExecuteFrameAction(lw_bar, "SetPoint", {"BOTTOMLEFT", UIParent, db.offsets.lw.x, db.offsets.lw.y})
		ProfessionBars:ExecuteFrameAction(lw_bar, "SetWidth", 332.5)
		ProfessionBars:ExecuteFrameAction(lw_movebutton, "SetHeight", 17.5)
		ProfessionBars:ExecuteFrameAction(lw_movebutton, "SetWidth", 17.5)
		ProfessionBars:ExecuteFrameAction(lw_movebutton, "SetScript", {"OnMouseDown", profbars_startdragging_lw})
		ProfessionBars:ExecuteFrameAction(lw_movebutton, "SetScript", {"OnMouseUp", profbars_stopdragging_lw})
		ProfessionBars:ExecuteFrameAction(lw_movebutton, "SetPoint", {"TOPRIGHT", profbars_leatherworking, "TOPRIGHT"})
		ProfessionBars:ExecuteFrameAction(lw_movebutton, "SetHighlightTexture", {"Interface\\Icons\\Spell_Arcane_PortalStormWind", 0.8})
		if not profbars_temp.buttonscreated then
			for i=1, 14 do
				CreateButton("profbars_lwbutton"..i, profbars_leatherworking);
			end
		end
		if db.mode == 2 then
			ProfessionBars:ExecuteFrameAction(lw_bar, "ClearAllPoints")
			if IsAddOnLoaded("AdvancedTradeSkillWindow") then
				ProfessionBars:ExecuteFrameAction(lw_bar, "SetPoint", {"TOP", ATSWFrame, "BOTTOM", 0, 10})
			elseif IsAddOnLoaded("GnomeWorks") then
				ProfessionBars:ExecuteFrameAction(lw_bar, "SetPoint", {"TOP", GnomeWorksFrame, "BOTTOM", 0, -5})
			else
				ProfessionBars:ExecuteFrameAction(lw_bar, "SetPoint", {"TOP", TradeSkillFrame, "BOTTOMLEFT", 180, 75})
			end
			ProfessionBars:ExecuteFrameAction(lw_movebutton, "Hide")
			ProfessionBars:ExecuteFrameAction(lw_bar, "Hide")
		end
		
		-- We have a profession
		haveProfession = true;
	end

	if PlayerHasSpell(2575) then
		-- Mining
		
		-- We have a profession
		--haveProfession = true;
	end

	if PlayerHasSpell(8613) then
		-- Skinning
		
		-- We have a profession
		--haveProfession = true;
	end

	if PlayerHasSpell(3908) then
		-- Tailoring
		
		-- We have a profession
		--haveProfession = true;
	end
	
	if haveProfession then
		-- We have some profession, schedule a push of the items in 1 second
		ProfessionBars:ScheduleTimer("ITEM_PUSH", 1);
	end
	profbars_temp.buttonscreated = true;
end

------------------------------------------------------------------------------------------------------
-- What happens when we start dragging the Enchanting frame
------------------------------------------------------------------------------------------------------
function profbars_startdragging_ench()
	profbars_enchanting:StartMoving();
end

------------------------------------------------------------------------------------------------------
-- What happens when we stop dragging the Enchanting frame
------------------------------------------------------------------------------------------------------
function profbars_stopdragging_ench()
	db.offsets.ench.x = profbars_enchanting:GetLeft();
	db.offsets.ench.y = profbars_enchanting:GetBottom();
	profbars_enchanting:StopMovingOrSizing();
end

------------------------------------------------------------------------------------------------------
-- What happens when we start dragging the JC frame
------------------------------------------------------------------------------------------------------
function profbars_startdragging_jc()
	profbars_jewelcrafting:StartMoving()
end

------------------------------------------------------------------------------------------------------
-- What happens when we stop dragging the JC frame
------------------------------------------------------------------------------------------------------
function profbars_stopdragging_jc()
	db.offsets.jc.x = profbars_jewelcrafting:GetLeft();
	db.offsets.jc.y = profbars_jewelcrafting:GetBottom();
	profbars_jewelcrafting:StopMovingOrSizing();
end

------------------------------------------------------------------------------------------------------
-- What happens when we start dragging the Inscription frame
------------------------------------------------------------------------------------------------------
function profbars_startdragging_insc()
	profbars_inscription:StartMoving()
end

------------------------------------------------------------------------------------------------------
-- What happens when we stop dragging the Inscription frame
------------------------------------------------------------------------------------------------------
function profbars_stopdragging_insc()
	db.offsets.insc.x = profbars_inscription:GetLeft();
	db.offsets.insc.y = profbars_inscription:GetBottom();
	profbars_inscription:StopMovingOrSizing();
end

------------------------------------------------------------------------------------------------------
-- What happens when we start dragging the Leatherworking frame
------------------------------------------------------------------------------------------------------
function profbars_startdragging_lw()
	profbars_leatherworking:StartMoving()
end

------------------------------------------------------------------------------------------------------
-- What happens when we stop dragging the Leatherworking frame
------------------------------------------------------------------------------------------------------
function profbars_stopdragging_lw()
	db.offsets.lw.x = profbars_leatherworking:GetLeft();
	db.offsets.lw.y = profbars_leatherworking:GetBottom();
	profbars_leatherworking:StopMovingOrSizing();
end

------------------------------------------------------------------------------------------------------
-- What happens when we start dragging the Blacksmithing frame
------------------------------------------------------------------------------------------------------
function profbars_startdragging_bs()
	profbars_blacksmithing:StartMoving()
end

------------------------------------------------------------------------------------------------------
-- What happens when we stop dragging the Blacksmithing frame
------------------------------------------------------------------------------------------------------
function profbars_stopdragging_bs()
	db.offsets.bs.x = profbars_blacksmithing:GetLeft();
	db.offsets.bs.y = profbars_blacksmithing:GetBottom();
	profbars_blacksmithing:StopMovingOrSizing();
end

------------------------------------------------------------------------------------------------------
-- We've gained a new skill level		UNTESTED!
------------------------------------------------------------------------------------------------------
function ProfessionBars:CHAT_MSG_SKILL(event, msg)
	if string.match(msg, (GetSpellInfo(25229))) or string.match(msg, (GetSpellInfo(45357))) or string.match(msg, (GetSpellInfo(2108))) or string.match(msg, (GetSpellInfo(7411))) or string.match(msg, (GetSpellInfo(2656))) or string.match(msg, (GetSpellInfo(2018))) then
		ProfessionBars:ITEM_PUSH();
	end
end

------------------------------------------------------------------------------------------------------
-- What happens when we open tradeskill window
------------------------------------------------------------------------------------------------------
function ProfessionBars:TRADE_SKILL_SHOW()
	if not IsTradeSkillLinked() then
		if db.mode == 1 then
			return;
		end

		local a = GetTradeSkillLine();
		if string.match(a, "%S+") == GetSpellInfo(25229) then
			ProfessionBars:ExecuteFrameAction(profbars_jewelcrafting, "ClearAllPoints")
			if IsAddOnLoaded("AdvancedTradeSkillWindow") then
				ProfessionBars:ExecuteFrameAction(profbars_jewelcrafting, "SetPoint", {"TOP", ATSWFrame, "BOTTOM", 0, 10})
			elseif IsAddOnLoaded("GnomeWorks") then
				ProfessionBars:ExecuteFrameAction(profbars_jewelcrafting, "SetPoint", {"TOP", GnomeWorksFrame, "BOTTOM", 0, -5})
			else
				ProfessionBars:ExecuteFrameAction(profbars_jewelcrafting, "SetPoint", {"TOP", TradeSkillFrame, "BOTTOMLEFT", 180, 75})
			end
			if PlayerHasSpell(45357) then
				ProfessionBars:ExecuteFrameAction(profbars_inscription, "Hide")
			end
			if PlayerHasSpell(2108) then
				ProfessionBars:ExecuteFrameAction(profbars_leatherworking, "Hide")
			end
			if PlayerHasSpell(7411) then
				ProfessionBars:ExecuteFrameAction(profbars_enchanting, "Hide")
			end
			if PlayerHasSpell(2018) and PlayerHasSpell(2656) then
				ProfessionBars:ExecuteFrameAction(profbars_blacksmithing, "Hide")
			end
			ProfessionBars:ExecuteFrameAction(profbars_jewelcrafting, "Show")
		elseif string.match(a, "%S+") == GetSpellInfo(45357) then
			ProfessionBars:ExecuteFrameAction(profbars_inscription, "ClearAllPoints")
			if IsAddOnLoaded("AdvancedTradeSkillWindow") then
				ProfessionBars:ExecuteFrameAction(profbars_inscription, "SetPoint", {"TOP", ATSWFrame, "BOTTOM", 0, 10})
			elseif IsAddOnLoaded("GnomeWorks") then
				ProfessionBars:ExecuteFrameAction(profbars_inscription, "SetPoint", {"TOP", GnomeWorksFrame, "BOTTOM", 0, -5})
			else
				ProfessionBars:ExecuteFrameAction(profbars_inscription, "SetPoint", {"TOP", TradeSkillFrame, "BOTTOMLEFT", 180, 75})
			end
			if PlayerHasSpell(25229) then
				ProfessionBars:ExecuteFrameAction(profbars_jewelcrafting, "Hide")
			end
			if PlayerHasSpell(2108) then
				ProfessionBars:ExecuteFrameAction(profbars_leatherworking, "Hide")
			end
			if PlayerHasSpell(7411) then
				ProfessionBars:ExecuteFrameAction(profbars_enchanting, "Hide")
			end
			if PlayerHasSpell(2018) and PlayerHasSpell(2656) then
				ProfessionBars:ExecuteFrameAction(profbars_blacksmithing, "Hide")
			end
			ProfessionBars:ExecuteFrameAction(profbars_inscription, "Show")
		elseif string.match(a, "%S+") == GetSpellInfo(2108) then
			ProfessionBars:ExecuteFrameAction(profbars_leatherworking, "ClearAllPoints")
			if IsAddOnLoaded("AdvancedTradeSkillWindow") then
				ProfessionBars:ExecuteFrameAction(profbars_leatherworking, "SetPoint", {"TOP", ATSWFrame, "BOTTOM", 0, 10})
			elseif IsAddOnLoaded("GnomeWorks") then
				ProfessionBars:ExecuteFrameAction(profbars_leatherworking, "SetPoint", {"TOP", GnomeWorksFrame, "BOTTOM", 0, -5})
			else
				ProfessionBars:ExecuteFrameAction(profbars_leatherworking, "SetPoint", {"TOP", TradeSkillFrame, "BOTTOMLEFT", 180, 75})
			end
			if PlayerHasSpell(25229) then
				ProfessionBars:ExecuteFrameAction(profbars_jewelcrafting, "Hide")
			end
			if PlayerHasSpell(45357) then
				ProfessionBars:ExecuteFrameAction(profbars_inscription, "Hide")
			end
			if PlayerHasSpell(7411) then
				ProfessionBars:ExecuteFrameAction(profbars_enchanting, "Hide")
			end
			if PlayerHasSpell(2018) and PlayerHasSpell(2656) then
				ProfessionBars:ExecuteFrameAction(profbars_blacksmithing, "Hide")
			end
			ProfessionBars:ExecuteFrameAction(profbars_leatherworking, "Show")
		elseif string.match(a, "%S+") == GetSpellInfo(7411) then
			ProfessionBars:ExecuteFrameAction(profbars_enchanting, "ClearAllPoints")
			if IsAddOnLoaded("AdvancedTradeSkillWindow") then
				ProfessionBars:ExecuteFrameAction(profbars_enchanting, "SetPoint", {"TOP", ATSWFrame, "BOTTOM", 0, 10})
			elseif IsAddOnLoaded("GnomeWorks") then
				ProfessionBars:ExecuteFrameAction(profbars_enchanting, "SetPoint", {"TOP", GnomeWorksFrame, "BOTTOM", 0, -5})
			else
				ProfessionBars:ExecuteFrameAction(profbars_enchanting, "SetPoint", {"TOP", TradeSkillFrame, "BOTTOMLEFT", 180, 75})
			end
			if PlayerHasSpell(25229) then
				ProfessionBars:ExecuteFrameAction(profbars_jewelcrafting, "Hide")
			end
			if PlayerHasSpell(45357) then
				ProfessionBars:ExecuteFrameAction(profbars_inscription, "Hide")
			end
			if PlayerHasSpell(2108) then
				ProfessionBars:ExecuteFrameAction(profbars_leatherworking, "Hide")
			end
			if PlayerHasSpell(2018) and PlayerHasSpell(2656) then
				ProfessionBars:ExecuteFrameAction(profbars_blacksmithing, "Hide")
			end
			ProfessionBars:ExecuteFrameAction(profbars_enchanting, "Show")
		elseif string.match(a, "%S+") == GetSpellInfo(2018) then
			ProfessionBars:ExecuteFrameAction(profbars_blacksmithing, "ClearAllPoints")
			if IsAddOnLoaded("AdvancedTradeSkillWindow") then
				ProfessionBars:ExecuteFrameAction(profbars_blacksmithing, "SetPoint", {"TOP", ATSWFrame, "BOTTOM", 0, 10})
			elseif IsAddOnLoaded("GnomeWorks") then
				ProfessionBars:ExecuteFrameAction(profbars_blacksmithing, "SetPoint", {"TOP", GnomeWorksFrame, "BOTTOM", 0, -5})
			else
				ProfessionBars:ExecuteFrameAction(profbars_blacksmithing, "SetPoint", {"TOP", TradeSkillFrame, "BOTTOMLEFT", 180, 75})
			end
			if PlayerHasSpell(25229) then
				ProfessionBars:ExecuteFrameAction(profbars_jewelcrafting, "Hide")
			end
			if PlayerHasSpell(45357) then
				ProfessionBars:ExecuteFrameAction(profbars_inscription, "Hide")
			end
			if PlayerHasSpell(2108) then
				ProfessionBars:ExecuteFrameAction(profbars_leatherworking, "Hide")
			end
			if PlayerHasSpell(7411) then
				ProfessionBars:ExecuteFrameAction(profbars_enchanting, "Hide")
			end
			ProfessionBars:ExecuteFrameAction(profbars_blacksmithing, "Show")
		else
			if PlayerHasSpell(25229) then
				ProfessionBars:ExecuteFrameAction(profbars_jewelcrafting, "Hide")
			end
			if PlayerHasSpell(45357) then
				ProfessionBars:ExecuteFrameAction(profbars_inscription, "Hide")
			end
			if PlayerHasSpell(2108) then
				ProfessionBars:ExecuteFrameAction(profbars_leatherworking, "Hide")
			end
			if PlayerHasSpell(7411) then
				ProfessionBars:ExecuteFrameAction(profbars_enchanting, "Hide")
			end
			if PlayerHasSpell(2018) and PlayerHasSpell(2656) then
				ProfessionBars:ExecuteFrameAction(profbars_blacksmithing, "Hide")
			end
		end
		ProfessionBars:ITEM_PUSH();
	end
end

------------------------------------------------------------------------------------------------------
-- What happens when we close tradeskill window
------------------------------------------------------------------------------------------------------
function ProfessionBars:TRADE_SKILL_CLOSE()
	if db.mode == 1 then
		return;
	end

	if PlayerHasSpell(25229) then
		ProfessionBars:ExecuteFrameAction(profbars_jewelcrafting, "Hide")
	end
	if PlayerHasSpell(45357) then
		ProfessionBars:ExecuteFrameAction(profbars_inscription, "Hide")
	end
	if PlayerHasSpell(2108) then
		ProfessionBars:ExecuteFrameAction(profbars_leatherworking, "Hide")
	end
	if PlayerHasSpell(7411) then
		ProfessionBars:ExecuteFrameAction(profbars_enchanting, "Hide")
	end
	if PlayerHasSpell(2018) and PlayerHasSpell(2656) then
		ProfessionBars:ExecuteFrameAction(profbars_blacksmithing, "Hide")
	end
end

-------------------------------------------------------------------------------------------------------
-- Mecha-function for checking what buttons to show, and shiz; this is called whenever the player loots
-------------------------------------------------------------------------------------------------------
function ProfessionBars:ITEM_PUSH()
	if PlayerHasSpell(25229) then
		local c = ProfessionBars:GetLevel(25229);
		if profbars_debug then
			print("Jewelcrafting level "..c);
		end
		local a = {};
		if GetItemCount(2770) >= 5 and c >= 20 then
			local b = #a+1;
			a[b] = profbars_jcbutton9;
		end
		if GetItemCount(2771) >= 5 and c >= 50 then
			local b = #a + 1;
			a[b] = profbars_jcbutton8;
		end
		if GetItemCount(2772) >= 5 and c >= 125 then
			local b = #a+1;
			a[b] = profbars_jcbutton7;
		end
		if GetItemCount(3858) >= 5 and c >= 175 then
			local b = #a+1;
			a[b] = profbars_jcbutton6;
		end
		if GetItemCount(10620) >= 5 and c >= 250 then
			local b = #a+1;
			a[b] = profbars_jcbutton5;
		end
		if GetItemCount(23424) >= 5 and c >= 275 then
			local b = #a+1;
			a[b] = profbars_jcbutton4;
		end
		if GetItemCount(23425) >= 5 and c >= 325 then
			local b = #a+1;
			a[b] = profbars_jcbutton1;
		end
		if GetItemCount(36909) >= 5 and c >= 350 then
			local b = #a+1;
			a[b] = profbars_jcbutton2;
		end
		if GetItemCount(36912) >= 5 and c >= 400 then
			local b = #a+1;
			a[b] = profbars_jcbutton3;
		end
		if GetItemCount(36910) >= 5 and c >= 450 then
			local b = #a+1;
			a[b] = profbars_jcbutton10;
		end
		if db.mode == 1 then
			ProfessionBars:ExecuteFrameAction(profbars_jewelcrafting, "SetWidth", #a * 35 + 17.5)
		else
			ProfessionBars:ExecuteFrameAction(profbars_jewelcrafting, "SetWidth", #a * 35)
		end

		for x=1, 10 do
			ProfessionBars:ExecuteFrameAction(_G["profbars_jcbutton"..x], "Hide")
		end
		ProfessionBars:ExecuteFrameAction(profbars_jcmove, "Hide")

		for i = 1, #a do
			ProfessionBars:ExecuteFrameAction(a[i], "ClearAllPoints")
			ProfessionBars:ExecuteFrameAction(a[i], "SetPoint", {"TOPLEFT", profbars_jewelcrafting, "TOPLEFT", (i - 1) * 35, 0})
			ProfessionBars:ExecuteFrameAction(a[i], "Show")
			if profbars_debug then
				print("Showing "..a[i]:GetName());
			end
		end
		if db.mode == 1 and #a > 0 then
			ProfessionBars:ExecuteFrameAction(profbars_jcmove, "Show")
		end
	end
	if PlayerHasSpell(2018) and PlayerHasSpell(2656) then
		--Blacksmithing
		local c = ProfessionBars:GetLevel(2575);
		if profbars_debug then
			print("Mining level "..c);
		end
		local a = {};
		if GetItemCount(2770) >= 1 and c >= 1 then
			local b = #a+1;
			a[b] = profbars_bsbutton1;
		end
		if GetItemCount(2771) >= 1 and c >= 65 then
			local b = #a+1;
			a[b] = profbars_bsbutton2;
		end
		if GetItemCount(2840) >= 1 and GetItemCount(3576) >= 1 and c >= 65 then
			local b = #a+1;
			a[b] = profbars_bsbutton3;
		end
		if GetItemCount(2775) >= 1 and c >= 75 then
			local b = #a+1;
			a[b] = profbars_bsbutton4;
		end
		if GetItemCount(2772) >= 1 and c >= 125 then
			local b = #a+1;
			a[b] = profbars_bsbutton5;
		end
		if GetItemCount(2776) >= 1 and c >= 155 then
			local b = #a+1;
			a[b] = profbars_bsbutton6;
		end
		if GetItemCount(3575) >= 1 and GetItemCount(3857) >= 1 and c >= 165 then
			local b = #a+1;
			a[b] = profbars_bsbutton7;
		end
		if GetItemCount(3858) >= 1 and c >= 175 then
			local b = #a+1;
			a[b] = profbars_bsbutton8;
		end
		if GetItemCount(7911) >= 1 and c >= 230 then
			local b = #a+1;
			a[b] = profbars_bsbutton9;
		end
		if GetItemCount(11370) >= 8 and c >= 230 then
			local b = #a+1;
			a[b] = profbars_bsbutton10;
		end
		if GetItemCount(10620) >= 1 and c >= 250 then
			local b = #a+1;
			a[b] = profbars_bsbutton11;
		end
		if GetItemCount(18562) >= 1 and GetItemCount(12360) >= 10 and GetItemCount(17010) >= 1 and GetItemCount(18567) >= 3 and c >= 300 then
			local b = #a+1;
			a[b] = profbars_bsbutton12;
		end
		if GetItemCount(23424) >= 2 and c >= 300 then
			local b = #a+1;
			a[b] = profbars_bsbutton13;
		end
		if GetItemCount(23425) >= 2 and c >= 325 then
			local b = #a+1;
			a[b] = profbars_bsbutton14;
		end
		if GetItemCount(23427) >= 2 and c >= 350 then
			local b = #a+1;
			a[b] = profbars_bsbutton15;
		end
		if GetItemCount(23445) >= 3 and GetItemCount(23447) >= 2 and c >= 350 then
			local b = #a+1;
			a[b] = profbars_bsbutton16;
		end
		if GetItemCount(23426) >= 2 and c >= 375 then
			local b = #a+1;
			a[b] = profbars_bsbutton17;
		end
		if GetItemCount(23446) >= 10 and c >= 375 then
			local b = #a+1;
			a[b] = profbars_bsbutton18;
		end
		if GetItemCount(23449) >= 3 and GetItemCount(23573) >= 1 and c >= 375 then
			local b = #a+1;
			a[b] = profbars_bsbutton19;
		end
		if GetItemCount(36909) >= 1 and c >= 350 then
			local b = #a+1;
			a[b] = profbars_bsbutton20;
		end
		if GetItemCount(36912) >= 2 and c >= 400 then
			local b = #a+1;
			a[b] = profbars_bsbutton21;
		end
		if GetItemCount(36910) >= 2 and c >= 450 then
			local b = #a+1;
			a[b] = profbars_bsbutton22;
		end
		if GetItemCount(41163) >= 3 and GetItemCount(36860) >= 1 and GetItemCount(35624) >= 1 and GetItemCount(35627) >= 1 and c >= 450 then
			local b = #a+1;
			a[b] = profbars_bsbutton23;
		end
		if db.mode == 1 then
			if #a > 9 then
				ProfessionBars:ExecuteFrameAction(profbars_blacksmithing, "SetWidth", 9*35+17.5);
			else
				ProfessionBars:ExecuteFrameAction(profbars_blacksmithing, "SetWidth", #a*35+17.5);
			end
		else
			if #a > 9 then
				ProfessionBars:ExecuteFrameAction(profbars_blacksmithing, "SetWidth", 9*35);
			else
				ProfessionBars:ExecuteFrameAction(profbars_blacksmithing, "SetWidth", #a*35);
			end
		end
		for i=1, 23 do
			ProfessionBars:ExecuteFrameAction(_G["profbars_bsbutton"..i], "Hide")
		end
		ProfessionBars:ExecuteFrameAction(profbars_bsmove, "Hide")
		if #a > 18 then
			ProfessionBars:ExecuteFrameAction(profbars_blacksmithing, "SetHeight", 105)
		elseif #a > 9 then
			ProfessionBars:ExecuteFrameAction(profbars_blacksmithing, "SetHeight", 70)
		else
			ProfessionBars:ExecuteFrameAction(profbars_blacksmithing, "SetHeight", 35)
		end
		for i=1, #a do
			local b = i;
			local vert
			if i > 18 then
				vert = -70;
				b = i-18;
			elseif i > 9 then
				vert = -35;
				b = i-9;
			else
				vert = 0;
				b = i;
			end
			ProfessionBars:ExecuteFrameAction(a[i], "ClearAllPoints")
			ProfessionBars:ExecuteFrameAction(a[i], "SetPoint", {"TOPLEFT", profbars_blacksmithing, "TOPLEFT", (b-1)*35, vert})
			ProfessionBars:ExecuteFrameAction(a[i], "Show")
			if profbars_debug then
				print("Showing "..a[i]:GetName());
			end
		end
		if db.mode == 1 and #a > 0 then
			ProfessionBars:ExecuteFrameAction(profbars_bsmove, "Show")
		end
	end
	if PlayerHasSpell(7411) then
		-- Enchanting
		local greens, blues, epixx = ProfessionBars:GetDisenchantableItems()
		local count = #greens+#blues+#epixx;
		if not ((#greens > 9) or (#blues > 9) or (#epixx > 9)) then
			local width = math.max(#greens, #blues, #epixx)*35;
			if db.mode == 1 then
				width = width+17.5;
			end
			profbars_enchanting:SetWidth(width);
			local height = 0;
			if #greens > 0 then	
				height = height+35;
			end
			if #blues > 0 then
				height = height+35;
			end
			if #epixx > 0 then
				height = height+35;
			end
			ProfessionBars:ExecuteFrameAction(profbars_enchanting, "SetHeight", height)
		else
			if db.mode == 1 then
				ProfessionBars:ExecuteFrameAction(profbars_enchanting, "SetWidth", 332.5)
			else
				ProfessionBars:ExecuteFrameAction(profbars_enchanting, "SetWidth", 315)
			end
			local height = 0;
			height = height+math.ceil(#greens/9)*35;
			height = height+math.ceil(#blues/9)*35;
			height = height+math.ceil(#epixx/9)*35;
			ProfessionBars:ExecuteFrameAction(profbars_enchanting, "SetHeight", height)
		end
		local hidecount = 1;
		while true do
			if _G["profbars_enchbutton"..hidecount] then
				ProfessionBars:ExecuteFrameAction(_G["profbars_enchbutton"..hidecount], "Hide")
				profbars_temp["enchantingindex"][hidecount] = {};
			else
				break;
			end
			hidecount = hidecount+1;
		end
		local counter = 1;
		profbars_temp["enchantingindex"] = {};
		for i=1, #greens do
			local btn;
			if not _G["profbars_enchbutton"..counter] then
				btn = CreateFrame("Button", "profbars_enchbutton"..counter, profbars_enchanting, "SecureActionButtonTemplate");
			else
				btn = _G["profbars_enchbutton"..counter];
			end
			ProfessionBars:ExecuteFrameAction(btn, "SetHeight", 35)
			ProfessionBars:ExecuteFrameAction(btn, "SetWidth", 35)
			ProfessionBars:ExecuteFrameAction(btn, "SetScript", {"PreClick", ButtonClick})
			ProfessionBars:ExecuteFrameAction(btn, "RegisterForClicks", "AnyUp")
			local offsety = math.floor(i/9)*-35;
			local offsetx = ((i-math.floor(i/9)*9)-1)*35;
			ProfessionBars:ExecuteFrameAction(btn, "SetPoint", {"TOPLEFT", profbars_enchanting, "TOPLEFT", offsetx, offsety})
			ProfessionBars:ExecuteFrameAction(btn, "SetNormalTexture", greens[i]["texture"])
			ProfessionBars:ExecuteFrameAction(btn, "SetHighlightTexture", {greens[i]["texture"], 1.5})
			ProfessionBars:ExecuteFrameAction(btn, "SetScript", {"OnEnter", OnEnter})
			ProfessionBars:ExecuteFrameAction(btn, "SetScript", {"OnLeave", OnLeave})
			ProfessionBars:ExecuteFrameAction(btn, "Show")
			profbars_temp["enchantingindex"][counter] = {};
			profbars_temp["enchantingindex"][counter]["link"] = greens[i]["link"];
			profbars_temp["enchantingindex"][counter]["bag"] = greens[i]["bag"];
			profbars_temp["enchantingindex"][counter]["slot"] = greens[i]["slot"];
			counter = counter + 1;
		end
		for i=1, #blues do
			local btn;
			if not _G["profbars_enchbutton"..counter] then
				btn = CreateFrame("Button", "profbars_enchbutton"..counter, profbars_enchanting, "SecureActionButtonTemplate");
			else
				btn = _G["profbars_enchbutton"..counter];
			end
			ProfessionBars:ExecuteFrameAction(btn, "SetHeight", 35)
			ProfessionBars:ExecuteFrameAction(btn, "SetWidth", 35)
			ProfessionBars:ExecuteFrameAction(btn, "SetScript", {"PreClick", ButtonClick})
			ProfessionBars:ExecuteFrameAction(btn, "RegisterForClicks", "AnyUp")
			local offsety = math.floor(i/9)*-35;
			offsety = offsety+math.ceil(#greens/9)*-35;
			local offsetx = ((i-math.floor(i/9)*9)-1)*35;
			ProfessionBars:ExecuteFrameAction(btn, "SetPoint", {"TOPLEFT", profbars_enchanting, "TOPLEFT", offsetx, offsety})
			ProfessionBars:ExecuteFrameAction(btn, "SetNormalTexture", blues[i]["texture"]);
			ProfessionBars:ExecuteFrameAction(btn, "SetHighlightTexture", {blues[i]["texture"], 1.5})
			ProfessionBars:ExecuteFrameAction(btn, "SetScript", {"OnEnter", OnEnter})
			ProfessionBars:ExecuteFrameAction(btn, "SetScript", {"OnLeave", OnLeave})
			ProfessionBars:ExecuteFrameAction(btn, "Show")
			profbars_temp["enchantingindex"][counter] = {};
			profbars_temp["enchantingindex"][counter]["link"] = blues[i]["link"];
			profbars_temp["enchantingindex"][counter]["bag"] = blues[i]["bag"];
			profbars_temp["enchantingindex"][counter]["slot"] = blues[i]["slot"];
			counter = counter+1;
		end
		for i=1, #epixx do
			local btn;
			if not _G["profbars_enchbutton"..counter] then
				btn = CreateFrame("Button", "profbars_enchbutton"..counter, profbars_enchanting, "SecureActionButtonTemplate");
			else
				btn = _G["profbars_enchbutton"..counter];
			end
			ProfessionBars:ExecuteFrameAction(btn, "SetHeight", 35)
			ProfessionBars:ExecuteFrameAction(btn, "SetWidth", 35)
			ProfessionBars:ExecuteFrameAction(btn, "SetScript", {"PreClick", ButtonClick})
			ProfessionBars:ExecuteFrameAction(btn, "RegisterForClicks", "AnyUp")
			local offsety = math.floor(i/9)*-35;
			offsety = offsety+math.ceil(#greens/9)*-35;
			offsety = offsety+math.ceil(#blues/9)*-35;
			local offsetx = ((i-math.floor(i/9)*9)-1)*35;
			ProfessionBars:ExecuteFrameAction(btn, "SetPoint", {"TOPLEFT", profbars_enchanting, "TOPLEFT", offsetx, offsety})
			ProfessionBars:ExecuteFrameAction(btn, "SetNormalTexture", epixx[i]["texture"])
			ProfessionBars:ExecuteFrameAction(btn, "SetHighlightTexture", {epixx[i]["texture"], 1.5})
			ProfessionBars:ExecuteFrameAction(btn, "SetScript", {"OnEnter", OnEnter})
			ProfessionBars:ExecuteFrameAction(btn, "SetScript", {"OnLeave", OnLeave})
			ProfessionBars:ExecuteFrameAction(btn, "Show")
			profbars_temp["enchantingindex"][counter] = {};
			profbars_temp["enchantingindex"][counter]["link"] = epixx[i]["link"];
			profbars_temp["enchantingindex"][counter]["bag"] = epixx[i]["bag"];
			profbars_temp["enchantingindex"][counter]["slot"] = epixx[i]["slot"];
			counter = counter+1;
		end
	end
	if PlayerHasSpell(45357) then
		-- Check Inscription stuffs
		
		local c = ProfessionBars:GetLevel(45357);
		if profbars_debug then
			print("ProfessionBarsDebug: Inscription level "..c);
		end
		local a = {};
		if ((GetItemCount(2449) >= 5) or (GetItemCount(2447) >= 5) or (GetItemCount(765) >= 5)) and c >= 1 then
			local b = #a+1;
			a[b] = profbars_inscbutton9;
		end
		if (((GetItemCount(2450)>=5) or (GetItemCount(2453)>=5) or (GetItemCount(3820)>=5) or (GetItemCount(2452)>=5)) and c >= 25) or (GetItemCount(785) >= 5 and c >= 1) then
			local b = #a+1;
			a[b] = profbars_inscbutton8;
		end
		if ((GetItemCount(3369) >= 5) or (GetItemCount(3356) >= 5) or (GetItemCount(3357) >= 5) or (GetItemCount(3355)>=5)) and c >= 75 then
			local b = #a+1;
			a[b] = profbars_inscbutton7;
		end
		if ((GetItemCount(3818) >= 5) or (GetItemCount(3821) >= 5) or (GetItemCount(3358) >= 5) or (GetItemCount(3819)>=5)) and c >= 125 then
			local b = #a+1;
			a[b] = profbars_inscbutton6;
		end
		if ((GetItemCount(8836) >= 5) or (GetItemCount(8839) >= 5) or (GetItemCount(4625)>=5) or (GetItemCount(8845)>=5) or (GetItemCount(8846)>=5) or (GetItemCount(8831) >= 5) or (GetItemCount(8838)>=5)) and c >= 175 then
			local b = #a+1;
			a[b] = profbars_inscbutton5;
		end
		if ((GetItemCount(13463) >= 5) or (GetItemCount(13464) >= 5) or (GetItemCount(13467) >= 5) or (GetItemCount(13465) >= 5) or (GetItemCount(13466) >= 5)) and c >= 225 then
			local b = #a+1;
			a[b] = profbars_inscbutton4;
		end
		if ((GetItemCount(22790)>=5) or (GetItemCount(22786) >= 5) or (GetItemCount(22785) >= 5) or (GetItemCount(22793) >= 5) or (GetItemCount(22791) >= 5) or (GetItemCount(22792)>=5) or (GetItemCount(22787)>=5) or (GetItemCount(22789)>=5)) and c >= 275 then
			local b = #a+1;
			a[b] = profbars_inscbutton1;
		end
		if ((GetItemCount(36903)>=5) or (GetItemCount(37921)>=5) or (GetItemCount(39970) >= 5) or (GetItemCount(36901)>=5) or (GetItemCount(36906)>=5) or (GetItemCount(36905)>=5) or (GetItemCount(36907)>=5) or (GetItemCount(36904)>=5)) and c >= 325 then
			local b = #a+1;
			a[b] = profbars_inscbutton2;
		end

		if db.mode == 1 then
			ProfessionBars:ExecuteFrameAction(profbars_inscription, "SetWidth", #a * 35 + 17.5);
		else
			ProfessionBars:ExecuteFrameAction(profbars_inscription, "SetWidth", #a * 35);
		end

		for x=1, 9 do
			if x ~= 3 then
				ProfessionBars:ExecuteFrameAction(_G["profbars_inscbutton"..x], "Hide")
			end
		end

		for i = 1, #a do
			ProfessionBars:ExecuteFrameAction(a[i], "ClearAllPoints")
			ProfessionBars:ExecuteFrameAction(a[i], "SetPoint", {"TOPLEFT", profbars_inscription, "TOPLEFT", (i - 1) * 35, 0})
			ProfessionBars:ExecuteFrameAction(a[i], "Show")
			if profbars_debug then
				print("Showing "..a[i]:GetName());
			end
		end

		if db.mode == 1 and #a > 0 then
			ProfessionBars:ExecuteFrameAction(profbars_inscmove, "Show")
		end
	end
	
	if PlayerHasSpell(2108) then
		-- Check Leatherworking stuffs
		
		local c = ProfessionBars:GetLevel(2108);
		if profbars_debug then
			print("ProfessionBarsDebug: Leatherworking level "..c);
		end
		local a = {};
		local d = {};
		if (GetItemCount(2934) >= 3) and c >= 1 then
			local b = #a+1;
			a[b] = profbars_lwbutton9;
		end
		if (GetItemCount(2318) >= 4) and c >= 100 then
			local b = #a+1;
			a[b] = profbars_lwbutton8;
		end
		if (GetItemCount(2319) >= 5) and c >= 150 then
			local b = #a+1;
			a[b] = profbars_lwbutton7;
		end
		if (GetItemCount(4234) >= 6) and c >= 200 then
			local b = #a+1;
			a[b] = profbars_lwbutton6;
		end
		if (GetItemCount(4304) >= 6) and c >= 250 then
			local b = #a+1;
			a[b] = profbars_lwbutton5;
		end
		if (GetItemCount(25649) >= 5) and c >= 300 then
			local b = #a+1;
			a[b] = profbars_lwbutton4;
		end
		if (GetItemCount(21887) >= 5) and c >= 325 then
			local b = #a+1;
			a[b] = profbars_lwbutton1;
		end
		if (GetItemCount(33567) >= 5) and c >= 350 then
			local b = #a+1;
			a[b] = profbars_lwbutton2;
		end
		if (GetItemCount(33568) >= 6) and c >= 390 then
			local b = #a+1;
			a[b] = profbars_lwbutton3;
		end
		if ((GetItemCount(783) >= 1) and (GetItemCount(4289) >= 1)) and c >= 35 then
			local e = #d+1;
			d[e] = profbars_lwbutton10;
		end
		if ((GetItemCount(4232) >= 1) and (GetItemCount(4289) >= 1)) and c >= 100 then
			local e = #d+1;
			d[e] = profbars_lwbutton11;
		end
		if ((GetItemCount(4235) >= 1) and (GetItemCount(4289) >= 3)) and c >= 150 then
			local e = #d+1;
			d[e] = profbars_lwbutton12;
		end
		if ((GetItemCount(8169) >= 1) and (GetItemCount(8150) >= 1)) and c >= 200 then
			local e = #d+1;
			d[e] = profbars_lwbutton13;
		end
		if ((GetItemCount(8171) >= 1) and (GetItemCount(15409) >= 1)) and c >= 250 then
			local e = #d+1;
			d[e] = profbars_lwbutton14;
		end
		if db.mode == 1 then
			if #a and #d then
				if #a > #d then
					ProfessionBars:ExecuteFrameAction(profbars_leatherworking, "SetWidth", #a*35+17.5)
				else
					ProfessionBars:ExecuteFrameAction(profbars_leatherworking, "SetWidth", #d*35+17.5)
				end
			else
				if #a then
					ProfessionBars:ExecuteFrameAction(profbars_leatherworking, "SetWidth", #a*35+17.5);
					ProfessionBars:ExecuteFrameAction(profbars_leatherworking, "SetHeight", 35);
				elseif #d then
					ProfessionBars:ExecuteFrameAction(profbars_leatherworking, "SetWidth", #d*35+17.5);
					ProfessionBars:ExecuteFrameAction(profbars_leatherworking, "SetHeight", 35);
				else
					ProfessionBars:ExecuteFrameAction(profbars_leatherworking, "SetWidth", 0);
					ProfessionBars:ExecuteFrameAction(profbars_leatherworking, "SetHeight", 0);
				end
			end
		else
			if #a and #d then
				if #a > #d then
					ProfessionBars:ExecuteFrameAction(profbars_leatherworking, "SetWidth", #a*35);
				else
					ProfessionBars:ExecuteFrameAction(profbars_leatherworking, "SetWidth", #d*35);
				end
			else
				if #a then
					ProfessionBars:ExecuteFrameAction(profbars_leatherworking, "SetWidth", #a*35);
					ProfessionBars:ExecuteFrameAction(profbars_leatherworking, "SetHeight", 35);
				elseif #d then
					ProfessionBars:ExecuteFrameAction(profbars_leatherworking, "SetWidth", #d*35);
					ProfessionBars:ExecuteFrameAction(profbars_leatherworking, "SetHeight", 35);
				else
					ProfessionBars:ExecuteFrameAction(profbars_leatherworking, "SetWidth", 0);
					ProfessionBars:ExecuteFrameAction(profbars_leatherworking, "SetHeight", 0);
				end
			end
		end
		for x=1, 14 do
			ProfessionBars:ExecuteFrameAction(_G["profbars_lwbutton"..x], "Hide")
		end
		
		for i = 1, #a do
			ProfessionBars:ExecuteFrameAction(a[i], "ClearAllPoints")
			ProfessionBars:ExecuteFrameAction(a[i], "SetPoint", {"TOPLEFT", profbars_leatherworking, "TOPLEFT", (i-1)*35, 0})
			ProfessionBars:ExecuteFrameAction(a[i], "Show")
			if profbars_debug then
				print("Showing "..a[i]:GetName());
			end
		end
		if #a then
			for i = 1, #d do
				ProfessionBars:ExecuteFrameAction(d[i], "ClearAllPoints")
				ProfessionBars:ExecuteFrameAction(d[i], "SetPoint", {"TOPLEFT", profbars_leatherworking, "TOPLEFT", (i-1)*35, 35})
				ProfessionBars:ExecuteFrameAction(d[i], "Show")
				if profbars_debug then
					print("Showing "..d[i]:GetName());
				end
			end
		else
			for i=1, #d do	
				ProfessionBars:ExecuteFrameAction(d[i], "ClearAllPoints")
				ProfessionBars:ExecuteFrameAction(d[i], "SetPoint", {"TOPLEFT", profbars_leatherworking, "TOPLEFT", (i-1)*35, 0})
				ProfessionBars:ExecuteFrameAction(d[i], "Show")
				if profbars_debug then
					print("Showing "..d[i]:GetName());
				end
			end
		end
		if db.mode == 1 and ((#a > 0) or (#d > 0)) then
			ProfessionBars:ExecuteFrameAction(profbars_lwmove, "Show")
		end
	end	
end
